@@ -8,22 +8,35 @@ from python_paypal_api.base.credential_provider import CredentialProvider
 from python_paypal_api.base.api_response import ApiResponse
 from python_paypal_api.base.base_client import BaseClient
 from python_paypal_api.base.enum import EndPoint
+from .exceptions import get_exception_for_content, get_exception_for_code
 import os
 
 log = logging.getLogger(__name__)
 
 class Client(BaseClient):
 
+    def __new__(
+            cls,
+            *args,
+            **kwargs
+    ):
+        if not hasattr(cls, 'instance'):
+            cls.instance = super(Client, cls).__new__(cls)
+        return cls.instance
+
+
     def __init__(
             self,
             credentials: str or dict = "default",
             store_credentials=True,
+            safe=True,
             proxies=None,
             verify=True,
             timeout=None,
             debug=False
     ):
 
+
         self.debug = debug
         self.credentials = CredentialProvider(
             credentials,
@@ -32,16 +45,17 @@ class Client(BaseClient):
 
         self.host = EndPoint[self.credentials.client_mode].value if self.credentials.client_mode is not None else EndPoint["SANDBOX"].value
         self.endpoint = self.scheme + self.host
-
         self.store_credentials = store_credentials
         self._auth = AccessTokenClient(
-            # account=account,
             credentials=self.credentials,
             store_credentials=self.store_credentials,
+            safe=safe,
             proxies=proxies,
             verify=verify,
             timeout=timeout,
+            debug=self.debug
         )
+
         self.timeout = timeout
         self.proxies = proxies
         self.verify = verify
@@ -51,9 +65,16 @@ class Client(BaseClient):
         return {
             'User-Agent': self.user_agent,
             'Authorization': 'Bearer %s' % self.auth.access_token,
-            # 'Content-Type': 'application/json'
         }
 
+    @property
+    def get_store_credentials(self):
+        return {
+            'Store-Credentials': self.store_credentials,
+            'End-Point': self.endpoint,
+            'Client-Id': self.credentials.client_id
+        }
+        return self.store_credentials
 
     @property
     def auth(self) -> AccessTokenResponse:
@@ -113,7 +134,7 @@ class Client(BaseClient):
 
         return self._check_response(res)
 
-    # @staticmethod
+
     def _check_response(self, res) -> ApiResponse:
 
         if self.debug:
@@ -123,7 +144,6 @@ class Client(BaseClient):
         headers = vars(res).get('headers')
         status_code = vars(res).get('status_code')
 
-
         if status_code == 204:
             data = None
         else:
@@ -167,5 +187,4 @@ class Client(BaseClient):
             exception = get_exception_for_code(vars(res).get('status_code'))
             raise exception(dictionary, headers=vars(res).get('headers'))
 
-        next_token = vars(res).get('_next')
-        return ApiResponse(data, next_token, headers=headers)
+        return ApiResponse(data, res.request.headers.get("Authorization")[7:], self.get_store_credentials, headers=headers)
